"""
Methods used to process the data and then calculate the corelation coefficients for multiple tables.

Author: Glebs Labunskis
Date: June 2024
"""

import pandas as pd

from point_biserial import calculate_point_biserial


def process_session_data(filter_value, file_path_session, file_path_theme):
    """
    Process session data and calculate the mean of response_value for each rand_id after filtering and merging with
    theme data.

    :param filter_value: str, value to filter response_type column
    :return: DataFrame, merged and processed data
    """

    # Any of the themes files could be used to properly join the tables
    table_theme = pd.read_csv(file_path_theme[0])

    # Read the CSV files into DataFrames
    table_session = pd.read_csv(file_path_session)

    # Ensure the response_value column is numeric
    table_session['response_value'] = pd.to_numeric(table_session['response_value'], errors='coerce')

    # Filter the table_session to only include rows with response_type as specified by filter_value
    filtered_table_session = table_session[table_session['response_type'] == filter_value]

    # Perform a right join on the common column
    result = table_theme[['rand_id']].merge(filtered_table_session, on='rand_id', how='left')

    # Group by rand_id and calculate the mean of the response_value column
    result = result.groupby('rand_id')['response_value'].mean().reset_index()

    return result


def calculate_point_biserial_correlations(fp_themes, df, binary_col, continuous_cols, output_file_path):
    """
    Calculate point-biserial correlations for the given themes and save the results to a CSV file.

    :param fp_themes: list of str, file paths to the theme data CSV files
    :param df: DataFrame, processed human state data
    :param binary_col: str, name of the binary column
    :param continuous_cols: list of str, names of the continuous columns
    :param output_file_path: str, path to save the output CSV file
    """
    results = []

    for fp in fp_themes:
        # Read the data from the CSV file
        data_frame_theme = pd.read_csv(fp)

        # Perform a right join on the common column
        data_frame = data_frame_theme.merge(df, on='rand_id', how='left')

        # Calculate point-biserial correlation for each continuous column
        for continuous_col in continuous_cols:
            try:
                r_pb, p_value = calculate_point_biserial(data_frame, binary_col, continuous_col)
                results.append({
                    'file': fp,
                    'r_pb': r_pb,
                    'p_value': p_value
                })
                print(f"File: {fp}, Continuous Column: {continuous_col}, r_pb: {r_pb}, p_value: {p_value}")
            except Exception as e:
                print(f"Error processing {fp} with column {continuous_col}: {e}")

    # Convert the results to a DataFrame
    results_df = pd.DataFrame(results)

    # Write the results to a CSV file
    results_df.to_csv(output_file_path, index=False, sep=",")
    print(f"Results have been written to {output_file_path}")
